// note; deleting the gadgetId is done in multiple places. This is to prevent one gadget from overwriting another. It seems like CKEditor uses the same object/gadgetId in some specific cases.
CKEDITOR.dialog.add( 'gadgetDialog', function( editor )
{
	var dialogLoader;
	function commitGadget( element, gadgetId )
	{
		var gadget = {};
		gadget.elements = [];

		// get current tab, and the elements in that tab
		var currentTabId = CKEDITOR.dialog.getCurrent().definition.dialog._.currentTabId;
		var tabElements = CKEDITOR.dialog.getCurrent().definition.dialog._.contents[ currentTabId ];

		element.removeAttributes(); // remove all attributes, because if the user changed tab/gadget, some attributes might not be needed (if they are needed, they will be added below)

		var calculatedHeight = 0;

		for( tabElement in tabElements )
		{
			if( tabElement != 'undefined' )
			{
				gadget.elements.push( { id: tabElement, value: tabElements[ tabElement ].getValue() } );
			}
		}

		if( calculatedHeight )
			element.setAttribute( 'height', calculatedHeight );

		gadget.name = CKEDITOR.dialog.getCurrent().definition.dialog._.currentTabId;

		// set id to element
		element.setText( '1' );

		// Refresh the fake image.
		var newFakeImage = editor.createFakeElement( element, 'cke_gadget ' + CKEDITOR.dialog.getCurrent().definition.dialog._.currentTabId, 'gadget', true );

		// Replace the current fake image if it exists, or add a new one
		if( this.fakeImage )
		{
			newFakeImage.replace( this.fakeImage );
			editor.getSelection().selectElement( newFakeImage );
		}
		else
			editor.insertElement( newFakeImage );

		// save complete. Hide dialog
		CKEDITOR.dialog.getCurrent().hide();
		delete this.gadgetId;

		if( dialogLoader )
			dialogLoader.remove();

		return true;
	}

	// retrieve the current gadget from the database
	function openGadget( realElement, fakeImage, gadgetId )
	{
		// demo, nothing here..
		return;
	}

	return {
		title: editor.lang.gadget.gadget,
		minWidth: 500,
		minHeight: 300,
		contents: [
			{
				id: 'tab-htmlcode',
				label: editor.lang.gadget.htmlCode,
				elements: [
					{
						type: 'textarea',
						id: 'htmlcode',
						label: editor.lang.gadget.htmlCode,
						rows: 13
					}
				]
			}
		],
		onShow: function()
		{
			// get the fake image and corresponding real element if there is one selected
			this.fakeImage = this.realElement = null;
			var fakeImage = this.getSelectedElement();
			if( fakeImage && fakeImage.data( 'cke-real-element-type' ) && fakeImage.data( 'cke-real-element-type' ) == 'gadget' )
			{
				this.fakeImage = fakeImage;
				this.realElement = editor.restoreRealElement( fakeImage );
				this.gadgetId = parseInt( this.realElement.getText(), 10 );

				if( this.realElement.getName() == 'div' ) // element seems to be a gadget. Initialize it.
					openGadget( this.realElement, this.fakeImage, this.gadgetId );
			}
			else
			{
				delete this.gadgetId;
			}

			if( dialogLoader )
			{
				dialogLoader.remove();
				dialogLoader = null;
				delete dialogLoader;
			}
		},
		onOk: function()
		{
			// if there is no fake image (new gadget), create one
			if( !this.fakeImage )
			{
				element = CKEDITOR.dom.element.createFromHtml( '<div class="gadgetPlugin ' + CKEDITOR.dialog.getCurrent().definition.dialog._.currentTabId + '"></div>', editor.document );
				element.setAttributes({
					width: 80,
					height: 80,
					title: editor.lang.gadget.gadget
				});
				delete this.gadgetId;
			}
			else
				element = this.realElement;

			// save the gadget in the database, and set the fake image.
			commitGadget( element, this.gadgetId );

			// prevent the dialog from closing now. We close the dialog manually when the gadget is saved in the database.
			return false;
		}
	};
});